

#ifndef __itkComposeUniformTransformsTransform_h
#define __itkComposeUniformTransformsTransform_h

#include <iostream>
#include "itkTransform.h"
#include "itkExceptionObject.h"
#include "itkMatrix.h"
#include "itkVectorContainer.h"


namespace itk
{

/** \brief Combines transformations of a vector space (e.g. space coordinates).
 *
 *	Iterates through a container TContainerType and successively transforms 
 *	space coordinates of input point by each transform in the container.
 *
 *	Transforms stored in a container MUST be derived from same type as ::Superclass, 
 *	i.e. Transform<TScalarType,NDimensions,NDimensions> where TScalarType and NDimensions
 *	are same as passed into this class template. Also specified in ::TransformsType.
 *
 *	TContainerType must conform to IndexedContainerInterface and must store transforms
 *	of type Transform< TScalarType, NDimensions, NDimensions >::Pointer (e.g. ::Superclass::Pointer).
 *
 * \author Jakub Bican, Department of Image Processing, Institute of Information Theory and Automation, Academy of Sciences of the Czech Republic.
 *
 * \ingroup Transforms
 */
template <
    class TScalarType = double,          // Data type for scalars (float or double)
    unsigned int NDimensions = 3,        // Number of dimensions
		class TContainerType = VectorContainer< int,typename Transform<TScalarType,NDimensions,NDimensions>::Pointer > >		 // Transforms container type
class ITK_EXPORT ComposeUniformTransformsTransform :
          public Transform< TScalarType, NDimensions, NDimensions >
{
public:
  /** Standard class typedefs. */
  typedef ComposeUniformTransformsTransform Self;
  typedef Transform< TScalarType, NDimensions, NDimensions > Superclass;
  typedef SmartPointer<Self>        Pointer;
  typedef SmartPointer<const Self>  ConstPointer;

  /** New macro for creation of through the object factory.*/
  itkNewMacro( Self );

  /** Run-time type information (and related methods). */
  itkTypeMacro( ComposeUniformTransformsTransform, Transform );

  /** Dimension of the domain space. */
  itkStaticConstMacro(SpaceDimension, unsigned int, NDimensions);
  itkStaticConstMacro(ParametersDimension, unsigned int, 0);

  /** Standard scalar type for this class. */
  typedef typename Superclass::ScalarType ScalarType;

  /** Standard Jacobian container. */
  typedef typename Superclass::JacobianType JacobianType;

  /** Standard vector type for this class. */
  typedef Vector<TScalarType, itkGetStaticConstMacro(SpaceDimension)> InputVectorType;
  typedef Vector<TScalarType, itkGetStaticConstMacro(SpaceDimension)> OutputVectorType;

  /** Standard covariant vector type for this class. */
  typedef CovariantVector<TScalarType, itkGetStaticConstMacro(SpaceDimension)> InputCovariantVectorType;
  typedef CovariantVector<TScalarType, itkGetStaticConstMacro(SpaceDimension)> OutputCovariantVectorType;

  /** Standard vnl_vector type for this class. */
  typedef vnl_vector_fixed<TScalarType, itkGetStaticConstMacro(SpaceDimension)> InputVnlVectorType;
  typedef vnl_vector_fixed<TScalarType, itkGetStaticConstMacro(SpaceDimension)> OutputVnlVectorType;

  /** Standard coordinate point type for this class. */
  typedef Point<TScalarType, itkGetStaticConstMacro(SpaceDimension)> InputPointType;
  typedef Point<TScalarType, itkGetStaticConstMacro(SpaceDimension)> OutputPointType;

	/** Container type for transforms - must conform to IndexedContainerInterface */
	typedef typename TContainerType ContainerType;

	/** Type of transforms to compose 
	 * 	
	 *	Here, the type should be taken from ContainerType::Element. (??!!)
	 *
	 */
	typedef Transform< TScalarType, itkGetStaticConstMacro(SpaceDimension), itkGetStaticConstMacro(SpaceDimension) > TransformType;

  /** Method to transform a point or a vector. */
  OutputPointType     TransformPoint(const InputPointType  &point ) const;
  virtual OutputVectorType TransformVector(const InputVectorType &) const;
  virtual OutputVnlVectorType TransformVector(const InputVnlVectorType &) const;
  virtual OutputCovariantVectorType TransformCovariantVector(
    const InputCovariantVectorType &) const;


  /** This method returns the permutation order of the PermuteAxesTransform. */
  itkSetConstObjectMacro(Transforms,ContainerType);


	
  /** This method returns the permutation order of the PermuteAxesTransform. */
  itkGetConstObjectMacro(Transforms,ContainerType);


  /** Compute the Jacobian Matrix of the transformation at one point - not applicable for this type of transform */
  virtual const JacobianType & GetJacobian(const InputPointType  &point ) const
		{
			itkExceptionMacro(<< "Method not applicable for composed transforms. ");
			return m_Jacobian;
		}

protected:
  ComposeUniformTransformsTransform();
  ~ComposeUniformTransformsTransform();

	/** Print contents of an TranslationTransform. */
  void PrintSelf(std::ostream &os, Indent indent) const;

private:
  ComposeUniformTransformsTransform(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

	typename ContainerType::ConstPointer m_Transforms;

}; //class ComposeUniformTransformsTransform


}  // namespace itk


#ifndef ITK_MANUAL_INSTANTIATION
#include "itkComposeUniformTransformsTransform.txx"
#endif

#endif /* __itkComposeUniformTransformsTransform_h */
